/*
 * ble_eddy.c
 *
 *  Created on: Nov 23, 2017
 *      Author: zbdn6r
 */

#include "app.h"

/* Global variable definition */
struct eddy_env_tag eddy_env;

/* ----------------------------------------------------------------------------
 * Function      : void Eddy_Env_Initialize(void)
 * ----------------------------------------------------------------------------
 * Description   : Initialize custom service environment
 * Inputs        : None
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
void EddyService_Env_Initialize(void) {
	/* Reset the application manager environment */
	memset(&eddy_env, 0, sizeof(eddy_env));

	eddy_env.capabilities_cccd_value = ATT_CCC_START_NTF;
	memcpy(eddy_env.capabilities_value,
			(uint8_t[EDDY_CAPABILITIES_MAX_LENGTH] ) { 0x00, 0x01, 0x00,
							0x00, 0x00, 0x02, 0x00, 0x03, 0x06 },
			EDDY_CAPABILITIES_MAX_LENGTH);
	eddy_env.advinterval_cccd_value = 0;
	eddy_env.lockstate_cccd_value = 0;
	eddy_env.lockstate_value[0] = 0x02;
	uint8_t lockkey[16] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xDE, 0xAD, 0xBE, 0xEF,	0xDE, 0xAD, 0xBE, 0xEF, 0xDE, 0xAD, 0xBE, 0xEF };

	memcpy(&eddy_env.lockstate_value[1], &lockkey, 16);
	uint8_t challenge[16] = { 0x00, 0x01, 0x02, 0x03, 0x00, 0x01, 0x02, 0x03,
			0x00, 0x01, 0x02, 0x03, 0x00, 0x01, 0x02, 0x03 };
	memcpy(eddy_env.challenge_value, &challenge, 16);
	eddy_env.remainconnectable_value = 0;
	eddy_env.advslotdata_length = 16;

	memcpy(eddy_env.advslotdata_value, (uint8_t[16] ) { 0x10, 0x03, 'o', 'n',
					's', 'e', 'm', 'i', '.', 'c', 'o', 'm', '/', 'i', 'd', 'k' },
			eddy_env.advslotdata_length);

//	memcpy(eddy_env.advslotdata_value, (uint8_t[16] ) { 0x10, 0x03, 'g', 'o',
//						'o', '.', 'g', 'l', '/', 'e', 'V', 'o', 'o', 'S', 'M' },
//		   eddy_env.advslotdata_length);


	eddy_env.advtxpower_value = OUTPUT_POWER_DBM; /* Set radio output power of RF */
	eddy_env.advinterval_value = APP_ADV_INT_MIN;

}

/* ----------------------------------------------------------------------------
 * Function      : void EddyService_ServiceAdd(void)
 * ----------------------------------------------------------------------------
 * Description   : Send request to add Eddy profile into the attribute database.
 *                 Defines the different access functions (setter/getter commands
 *                 to access the different characteristic attributes).
 * Inputs        : None
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
void EddyService_ServiceAdd(void) {
	struct gattm_add_svc_req *req = KE_MSG_ALLOC_DYN(GATTM_ADD_SVC_REQ,
			TASK_GATTM, TASK_APP, gattm_add_svc_req,
			EDDY_IDX_NB * sizeof(struct gattm_att_desc))
	;

	const uint8_t svc_uuid[ATT_UUID_128_LEN] = EDDY_SVC_UUID;

	const struct gattm_att_desc att[EDDY_IDX_NB] =
			{
			/* Attribute Index  = Attribute properties: UUID,
			 *                                          Permissions,
			 *                                          Max size,
			 *                                          Extra permissions */

			/* Capabilities Characteristic */
			[EDDY_IDX_CAPABILITIES_VALUE_CHAR] = ATT_DECL_CHAR(),
			[EDDY_IDX_CAPABILITIES_VALUE_VAL] =	 ATT_DECL_CHAR_UUID_128(EDDY_CHARACTERISTIC_CAPABILITIES_UUID, PERM(RD,ENABLE), EDDY_CAPABILITIES_MAX_LENGTH),
			[EDDY_IDX_CAPABILITIES_VALUE_CCC] =  ATT_DECL_CHAR_CCC(),
			[EDDY_IDX_CAPABILITIES_VALUE_USR_DSCP] = ATT_DECL_CHAR_USER_DESC(EDDY_USER_DESCRIPTION_MAX_LENGTH),

					/* Active Slot Characteristic */
					[EDDY_IDX_ACTIVESLOT_VALUE_CHAR] = ATT_DECL_CHAR(),
					[EDDY_IDX_ACTIVESLOT_VALUE_VAL
							] =
											ATT_DECL_CHAR_UUID_128(
													EDDY_CHARACTERISTIC_ACTIVESLOT_UUID,
													PERM(RD,ENABLE)| PERM(WRITE_REQ,ENABLE) | PERM(WRITE_COMMAND,ENABLE),
													1),
					[EDDY_IDX_ACTIVESLOT_VALUE_CCC] = ATT_DECL_CHAR_CCC(),
					[EDDY_IDX_ACTIVESLOT_VALUE_USR_DSCP] =
							ATT_DECL_CHAR_USER_DESC(1),

					/* Advertising Interval Characteristic */
					[EDDY_IDX_ADVINTERVAL_VALUE_CHAR] = ATT_DECL_CHAR(),
					[EDDY_IDX_ADVINTERVAL_VALUE_VAL
							] =
											ATT_DECL_CHAR_UUID_128(
													EDDY_CHARACTERISTIC_ADVINTERVAL_UUID,
													PERM(RD, ENABLE) | PERM(WRITE_REQ,ENABLE) | PERM(WRITE_COMMAND,ENABLE),
													2),
					[EDDY_IDX_ADVINTERVAL_VALUE_CCC] = ATT_DECL_CHAR_CCC(),
					[EDDY_IDX_ADVINTERVAL_VALUE_USR_DSCP] =
							ATT_DECL_CHAR_USER_DESC(
									EDDY_USER_DESCRIPTION_MAX_LENGTH),

					/* Radio Tx Power Characteristic */
					[EDDY_IDX_RADIOTXPOWER_VALUE_CHAR] = ATT_DECL_CHAR(),
					[EDDY_IDX_RADIOTXPOWER_VALUE_VAL
							] =
											ATT_DECL_CHAR_UUID_128(
													EDDY_CHARACTERISTIC_RADIOTXPOWER_UUID,
													PERM(RD,ENABLE)| PERM(WRITE_REQ,ENABLE) | PERM(WRITE_COMMAND,ENABLE),
													1),
					[EDDY_IDX_RADIOTXPOWER_VALUE_CCC] = ATT_DECL_CHAR_CCC(),
					[EDDY_IDX_RADIOTXPOWER_VALUE_USR_DSCP] =
							ATT_DECL_CHAR_USER_DESC(1),

					/* Advertised Tx Power Characteristic */
					[EDDY_IDX_ADVTXPOWER_VALUE_CHAR] = ATT_DECL_CHAR(),
					[EDDY_IDX_ADVTXPOWER_VALUE_VAL
							] =
											ATT_DECL_CHAR_UUID_128(
													EDDY_CHARACTERISTIC_ADVTXPOWER_UUID,
													PERM(RD,ENABLE)| PERM(WRITE_REQ,ENABLE) | PERM(WRITE_COMMAND,ENABLE),
													1),
					[EDDY_IDX_ADVTXPOWER_VALUE_CCC] = ATT_DECL_CHAR_CCC(),
					[EDDY_IDX_ADVTXPOWER_VALUE_USR_DSCP] =
							ATT_DECL_CHAR_USER_DESC(1),

					/* Lock State Characteristic */
					[EDDY_IDX_LOCKSTATE_VALUE_CHAR] = ATT_DECL_CHAR(),
					[EDDY_IDX_LOCKSTATE_VALUE_VAL] = ATT_DECL_CHAR_UUID_128(EDDY_CHARACTERISTIC_LOCKSTATE_UUID, PERM(RD,ENABLE)| PERM(WRITE_REQ,ENABLE), EDDY_LOCKSTATE_MAX_LENGTH),
					[EDDY_IDX_LOCKSTATE_VALUE_CCC] = ATT_DECL_CHAR_CCC(),
					[EDDY_IDX_LOCKSTATE_VALUE_USR_DSCP] = ATT_DECL_CHAR_USER_DESC(EDDY_USER_DESCRIPTION_MAX_LENGTH),

					/* Unlock Characteristic */
					[EDDY_IDX_UNLOCK_VALUE_CHAR] = ATT_DECL_CHAR(),
					[EDDY_IDX_UNLOCK_VALUE_VAL] =  ATT_DECL_CHAR_UUID_128(EDDY_CHARACTERISTIC_UNLOCK_UUID, PERM(RD,ENABLE) | PERM(WRITE_REQ,ENABLE), 16),
					[EDDY_IDX_UNLOCK_VALUE_CCC] =  ATT_DECL_CHAR_CCC(),
					[EDDY_IDX_UNLOCK_VALUE_USR_DSCP] =	ATT_DECL_CHAR_USER_DESC(EDDY_USER_DESCRIPTION_MAX_LENGTH),

					/* Advertisment Slot Data Characteristic */
					[EDDY_IDX_ADVSLOTDATA_VALUE_CHAR] = ATT_DECL_CHAR(),
					[EDDY_IDX_ADVSLOTDATA_VALUE_VAL
							] =
											ATT_DECL_CHAR_UUID_128(
													EDDY_CHARACTERISTIC_ADVSLOTDATA_UUID,
													PERM(RD,ENABLE)| PERM(WRITE_REQ,ENABLE) | PERM(WRITE_COMMAND,ENABLE),
													EDDY_ADVSLOTDATA_MAX_LENGTH),
					[EDDY_IDX_ADVSLOTDATA_VALUE_CCC] = ATT_DECL_CHAR_CCC(),
					[EDDY_IDX_ADVSLOTDATA_VALUE_USR_DSCP] =
							ATT_DECL_CHAR_USER_DESC(
									EDDY_USER_DESCRIPTION_MAX_LENGTH),

					/* Remain Connectable Characteristic */
					[EDDY_IDX_REMAINCONNECTABLE_VALUE_CHAR] = ATT_DECL_CHAR(),
					[EDDY_IDX_REMAINCONNECTABLE_VALUE_VAL] =
							ATT_DECL_CHAR_UUID_128(
									EDDY_CHARACTERISTIC_REMAINCONNECTABLE_UUID,
									PERM(RD,ENABLE), 1),
					[EDDY_IDX_REMAINCONNECTABLE_VALUE_CCC
							] = ATT_DECL_CHAR_CCC(),
					[EDDY_IDX_REMAINCONNECTABLE_VALUE_USR_DSCP] =
							ATT_DECL_CHAR_USER_DESC(
									EDDY_USER_DESCRIPTION_MAX_LENGTH)

					};

	/* Fill the add eddy service message */
	req->svc_desc.start_hdl = 0;
	req->svc_desc.task_id = TASK_APP;
	req->svc_desc.perm = PERM(SVC_UUID_LEN, UUID_128);
	req->svc_desc.nb_att = EDDY_IDX_NB;

	memcpy(&req->svc_desc.uuid[0], &svc_uuid[0], ATT_UUID_128_LEN);

	for (uint8_t i = 0; i < EDDY_IDX_NB; i++) {
		memcpy(&req->svc_desc.atts[i], &att[i], sizeof(struct gattm_att_desc));
	}

	/* Send the message */
	ke_msg_send(req);
}

/* ----------------------------------------------------------------------------
 * Function      : int Eddy_GATTM_AddSvcRsp(ke_msg_id_t const msg_id,
 *                                     struct gattm_add_svc_rsp
 *                                     const *param,
 *                                     ke_task_id_t const dest_id,
 *                                     ke_task_id_t const src_id)
 * ----------------------------------------------------------------------------
 * Description   : Handle the response from adding a service in the attribute
 *                 database from the GATT manager
 * Inputs        : - msg_id     - Kernel message ID number
 *                 - param      - Message parameters in format of
 *                                struct gattm_add_svc_rsp
 *                 - dest_id    - Destination task ID number
 *                 - src_id     - Source task ID number
 * Outputs       : return value - Indicate if the message was consumed;
 *                                compare with KE_MSG_CONSUMED
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
int Eddy_GATTM_AddSvcRsp(ke_msg_id_t const msg_id,
		struct gattm_add_svc_rsp const *param, ke_task_id_t const dest_id,
		ke_task_id_t const src_id) {
	eddy_env.start_hdl = param->start_hdl;

	/* Add the next requested service  */
	if (!Service_Add()) {
		/* All services have been added, go to the ready state
		 * and start advertising */
		ble_env.state = APPM_READY;
		Advertising_Start();
	}

	return (KE_MSG_CONSUMED);
}

/* ----------------------------------------------------------------------------
 * Function      : int Eddy_GATTC_ReadReqInd(ke_msg_id_t const msg_id,
 *                                      struct gattc_read_req_ind
 *                                      const *param,
 *                                      ke_task_id_t const dest_id,
 *                                      ke_task_id_t const src_id)
 * ----------------------------------------------------------------------------
 * Description   : Handle received read request indication
 *                 from a GATT Controller
 * Inputs        : - msg_id     - Kernel message ID number
 *                 - param      - Message parameters in format of
 *                                struct gattc_read_req_ind
 *                 - dest_id    - Destination task ID number
 *                 - src_id     - Source task ID number
 * Outputs       : return value - Indicate if the message was consumed;
 *                                compare with KE_MSG_CONSUMED
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
int Eddy_GATTC_ReadReqInd(ke_msg_id_t const msg_id,
		struct gattc_read_req_ind const *param, ke_task_id_t const dest_id,
		ke_task_id_t const src_id) {
	uint8_t length = 0;
	uint8_t status = GAP_ERR_NO_ERROR;
	uint16_t attnum;
	uint8_t *valptr = NULL;
	uint16_t *valptr_advint = NULL;

	struct gattc_read_cfm *cfm;

	/* Set the attribute handle using the attribute index
	 * in the custom service */
	if (param->handle > eddy_env.start_hdl) {
		attnum = (param->handle - eddy_env.start_hdl - 1);
	} else {
		status = ATT_ERR_INVALID_HANDLE;
	}

	/* If there is no error, send back the requested attribute value */
	if (status == GAP_ERR_NO_ERROR) {
		switch (attnum) {
		case EDDY_IDX_CAPABILITIES_VALUE_VAL:
			length = EDDY_CAPABILITIES_MAX_LENGTH;
			valptr = (uint8_t *) &eddy_env.capabilities_value;
			break;
		case EDDY_IDX_CAPABILITIES_VALUE_CCC:
			length = 2;
			valptr = (uint8_t *) &eddy_env.capabilities_cccd_value;
			break;
		case EDDY_IDX_CAPABILITIES_VALUE_USR_DSCP:
			length = strlen(EDDY_CAPABILITIES_CHARACTERISTIC_NAME);
			valptr = (uint8_t *) EDDY_CAPABILITIES_CHARACTERISTIC_NAME;
			break;
		case EDDY_IDX_ACTIVESLOT_VALUE_VAL:
			length = 1;
			valptr = (uint8_t *) &eddy_env.activeslot_value;
			break;
		case EDDY_IDX_ACTIVESLOT_VALUE_CCC:
			length = 2;
			valptr = (uint8_t *) &eddy_env.activeslot_cccd_value;
			break;
		case EDDY_IDX_ACTIVESLOT_VALUE_USR_DSCP:
			length = strlen(EDDY_ACTIVESLOT_CHARACTERISTIC_NAME);
			valptr = (uint8_t *) EDDY_ACTIVESLOT_CHARACTERISTIC_NAME;
			break;
		case EDDY_IDX_ADVINTERVAL_VALUE_VAL:
			length = 2;
			valptr_advint = (uint16_t*) &eddy_env.advinterval_value;
			break;
		case EDDY_IDX_ADVINTERVAL_VALUE_CCC:
			length = 1;
			valptr = (uint8_t *) &eddy_env.advinterval_cccd_value;
			break;
		case EDDY_IDX_ADVINTERVAL_VALUE_USR_DSCP:
			length = strlen(EDDY_ADVINTERVAL_CHARACTERISTIC_NAME);
			valptr = (uint8_t *) EDDY_ADVINTERVAL_CHARACTERISTIC_NAME;
			break;
		case EDDY_IDX_RADIOTXPOWER_VALUE_VAL:
			length = 1;
			valptr = (uint8_t *) &eddy_env.radiotxpower_value;
			break;
		case EDDY_IDX_RADIOTXPOWER_VALUE_CCC:
			length = 2;
			valptr = (uint8_t *) &eddy_env.radiotxpower_cccd_value;
			break;
		case EDDY_IDX_RADIOTXPOWER_VALUE_USR_DSCP:
			length = strlen(EDDY_RADIOTXPOWER_CHARACTERISTIC_NAME);
			valptr = (uint8_t *) EDDY_RADIOTXPOWER_CHARACTERISTIC_NAME;
			break;
		case EDDY_IDX_ADVTXPOWER_VALUE_VAL:
			length = 1;
			valptr = (uint8_t *) &eddy_env.advtxpower_value;
			break;
		case EDDY_IDX_ADVTXPOWER_VALUE_CCC:
			length = 2;
			valptr = (uint8_t *) &eddy_env.advtxpower_cccd_value;
			break;
		case EDDY_IDX_ADVTXPOWER_VALUE_USR_DSCP:
			length = strlen(EDDY_ADVTXPOWER_CHARACTERISTIC_NAME);
			valptr = (uint8_t *) EDDY_ADVTXPOWER_CHARACTERISTIC_NAME;
			break;
		case EDDY_IDX_LOCKSTATE_VALUE_VAL:
			length = 1;
			valptr = (uint8_t *) &eddy_env.lockstate_value;
			break;
		case EDDY_IDX_LOCKSTATE_VALUE_CCC:
			length = 2;
			valptr = (uint8_t *) &eddy_env.lockstate_cccd_value;
			break;
		case EDDY_IDX_LOCKSTATE_VALUE_USR_DSCP:
			length = strlen(EDDY_LOCKSTATE_CHARACTERISTIC_NAME);
			valptr = (uint8_t *) EDDY_LOCKSTATE_CHARACTERISTIC_NAME;
			break;
		case EDDY_IDX_UNLOCK_VALUE_VAL:
			length = 16;
			valptr = (uint8_t *) &eddy_env.challenge_value[0];
			break;
		case EDDY_IDX_UNLOCK_VALUE_CCC:
			length = 2;
			valptr = (uint8_t *) &eddy_env.challenge_cccd_value;
			break;
		case EDDY_IDX_UNLOCK_VALUE_USR_DSCP:
			length = strlen(EDDY_UNLOCK_CHARACTERISTIC_NAME);
			valptr = (uint8_t *) EDDY_UNLOCK_CHARACTERISTIC_NAME;
			break;
		case EDDY_IDX_ADVSLOTDATA_VALUE_VAL:
			length = eddy_env.advslotdata_length;
			valptr = (uint8_t *) &eddy_env.advslotdata_value;
			break;
		case EDDY_IDX_ADVSLOTDATA_VALUE_CCC:
			length = 2;
			valptr = (uint8_t *) &eddy_env.advslotdata_cccd_value;
			break;
		case EDDY_IDX_ADVSLOTDATA_VALUE_USR_DSCP:
			length = strlen(EDDY_ADVSLOTDATA_CHARACTERISTIC_NAME);
			valptr = (uint8_t *) EDDY_ADVSLOTDATA_CHARACTERISTIC_NAME;
			break;
		case EDDY_IDX_REMAINCONNECTABLE_VALUE_VAL:
			length = 1;
			valptr = (uint8_t *) &eddy_env.remainconnectable_value;
			break;
		case EDDY_IDX_REMAINCONNECTABLE_VALUE_CCC:
			length = 2;
			valptr = (uint8_t *) &eddy_env.remainconnectable_cccd_value;
			break;
		case EDDY_IDX_REMAINCONNECTABLE_VALUE_USR_DSCP:
			length = strlen(EDDY_REMAINCONNECTABLE_CHARACTERISTIC_NAME);
			valptr = (uint8_t *) EDDY_REMAINCONNECTABLE_CHARACTERISTIC_NAME;
			break;
		default:
			status = ATT_ERR_READ_NOT_PERMITTED;
			break;
		}
	}

	/* Allocate and build message */
	cfm = KE_MSG_ALLOC_DYN(GATTC_READ_CFM,
			KE_BUILD_ID(TASK_GATTC, ble_env.conidx), TASK_APP, gattc_read_cfm,
			length)
	;

	if (valptr != NULL) {
		memcpy(cfm->value, valptr, length);
	}

	if (valptr_advint != NULL) {
		uint8_t new_value[] = { (uint8_t) (*valptr_advint >> 8),
				(uint8_t) *valptr_advint };
		memcpy(cfm->value, &new_value, length);
	}

	cfm->handle = param->handle;
	cfm->length = length;
	cfm->status = status;

	/* Send the message */
	ke_msg_send(cfm);

	return (KE_MSG_CONSUMED);
}

/* ----------------------------------------------------------------------------
 * Function      : int GATTC_WriteReqInd(ke_msg_id_t const msg_id,
 *                                       struct gattc_write_req_ind
 *                                       const *param,
 *                                       ke_task_id_t const dest_id,
 *                                       ke_task_id_t const src_id)
 * ----------------------------------------------------------------------------
 * Description   : Handle received write request indication
 *                 from a GATT Controller
 * Inputs        : - msg_id     - Kernel message ID number
 *                 - param      - Message parameters in format of
 *                                struct gattc_write_req_ind
 *                 - dest_id    - Destination task ID number
 *                 - src_id     - Source task ID number
 * Outputs       : return value - Indicate if the message was consumed;
 *                                compare with KE_MSG_CONSUMED
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
int Eddy_GATTC_WriteReqInd(ke_msg_id_t const msg_id,
		struct gattc_write_req_ind const *param, ke_task_id_t const dest_id,
		ke_task_id_t const src_id) {
	struct gattc_write_cfm *cfm = KE_MSG_ALLOC(GATTC_WRITE_CFM,
			KE_BUILD_ID(TASK_GATTC, ble_env.conidx), TASK_APP, gattc_write_cfm);

	uint8_t status = GAP_ERR_NO_ERROR;
	uint16_t attnum;
	uint8_t *valptr = NULL;

	/* Check that offset is not zero */
	if (param->offset) {
		status = ATT_ERR_INVALID_OFFSET;
	}

	/* Set the attribute handle using the attribute index
	 * in the custom service */
	if (param->handle > eddy_env.start_hdl) {
		attnum = (param->handle - eddy_env.start_hdl - 1);
	} else {
		status = ATT_ERR_INVALID_HANDLE;
	}

	/* If there is no error, save the requested attribute value */
	if (status == GAP_ERR_NO_ERROR) {
		switch (attnum) {
		case EDDY_IDX_ACTIVESLOT_VALUE_VAL:
			valptr = (uint8_t *) &eddy_env.activeslot_value;
			break;
		case EDDY_IDX_LOCKSTATE_VALUE_VAL:
			if (param->length == 1 || param->length == 17)
			{
				valptr = (uint8_t *) &eddy_env.lockstate_value;
			}
			else
				status = ATT_ERR_WRITE_NOT_PERMITTED;
			break;
		case EDDY_IDX_UNLOCK_VALUE_VAL:
			valptr = (uint8_t *) &eddy_env.unlocktoken_value;
			break;
		case EDDY_IDX_ADVSLOTDATA_VALUE_VAL:
			valptr = (uint8_t *) &eddy_env.advslotdata_value;
			eddy_env.advslotdata_length = param->length;
			break;
		case EDDY_IDX_ADVTXPOWER_VALUE_VAL:
			valptr = (uint8_t *) &eddy_env.advtxpower_value;
			break;
		case EDDY_IDX_ADVINTERVAL_VALUE_VAL:
			valptr = (uint8_t *) &eddy_env.advinterval_value;
			break;
		default:
			status = ATT_ERR_WRITE_NOT_PERMITTED;
			break;
		}
	}

	if (valptr != NULL) {
		memcpy(valptr, param->value, param->length);

		if (attnum == EDDY_IDX_ADVINTERVAL_VALUE_VAL) {
			uint8_t new_value = eddy_env.advinterval_value;
			eddy_env.advinterval_value = (eddy_env.advinterval_value >> 8)
					| (new_value << 8);
		}

		if (attnum == EDDY_IDX_UNLOCK_VALUE_VAL) {

			/* Enable and configure the base band block */
			BBIF->CTRL = BB_CLK_ENABLE | BBCLK_DIVIDER_8 | BB_WAKEUP;
			/* Copy in the exchange memory */
			uint8_t plain_text[16];
			for (int i = 0; i<=15;i++)
				plain_text[i] = eddy_env.challenge_value[15-i];
			memcpy((void *) (EM_BLE_ENC_PLAIN_OFFSET + EM_BASE_ADDR), plain_text, 16);
			/* Configure the AES-128 engine for ciphering with the key and the memory
			 * zone */
			uint8_t encryptionkey[16];
			for (int i = 0; i<=15;i++)
							encryptionkey[i] = eddy_env.lockstate_value[16-i];
			Sys_AES_Config((void *) encryptionkey, EM_BLE_ENC_PLAIN_OFFSET);
			/* Run AES-128 encryption block */
			Sys_AES_Cipher();
			/* Access to the cipher-text at EM_BLE_ENC_CIPHER_OFFSET address */
			uint8_t encryptedtext_temp[16];
			memcpy(&encryptedtext_temp[0], (void *) (EM_BLE_ENC_CIPHER_OFFSET + EM_BASE_ADDR), 16);
			uint8_t encryptedtext[16];
			for (int i = 0; i<=15;i++)
				encryptedtext[i] = encryptedtext_temp[15-i];
			if (!memcmp(encryptedtext, eddy_env.unlocktoken_value, 16))
			{
				eddy_env.lockstate_value[0] = 0x01;
			}
		}
	}

//    if(attnum == EDDY_IDX_ADVSLOTDATA_VALUE_VAL)
//    {
//    	if (memcmp (&eddy_env.unlock_value[0], &eddy_env.lockstate_value[1], 16) == 0)
//    	{
//    		eddy_env.lockstate_value[0] = 0x01;
//    	}
//    }

	cfm->handle = param->handle;
	cfm->status = status;

	/* Send the message */
	ke_msg_send(cfm);

	return (KE_MSG_CONSUMED);
}

