/*
 * ble_eddy.h
 *
 *  Created on: Nov 22, 2017
 *      Author: zbdn6r
 */

#ifndef BLE_EDDY_H
#define BLE_EDDY_H

/* ----------------------------------------------------------------------------
 * If building with a C++ compiler, make all of the definitions in this header
 * have a C binding.
 * ------------------------------------------------------------------------- */
#ifdef __cplusplus
extern "C"
{
#endif

/* ----------------------------------------------------------------------------
 * Include files
 * --------------------------------------------------------------------------*/

/* ----------------------------------------------------------------------------
 * Defines
 * --------------------------------------------------------------------------*/

/* Custom service UUIDs */
#define EDDY_SVC_UUID                  			   		{0x95,0xe2,0xed,0xeb,0x1b,0xa0,0x39,0x8a,0xdf,0x4b,0xd3,0x8e,0x00,0x75,0xc8,0xa3}
#define EDDY_CHARACTERISTIC_CAPABILITIES_UUID       	{0x95,0xe2,0xed,0xeb,0x1b,0xa0,0x39,0x8a,0xdf,0x4b,0xd3,0x8e,0x01,0x75,0xc8,0xa3}
#define EDDY_CHARACTERISTIC_ACTIVESLOT_UUID				{0x95,0xe2,0xed,0xeb,0x1b,0xa0,0x39,0x8a,0xdf,0x4b,0xd3,0x8e,0x02,0x75,0xc8,0xa3}
#define EDDY_CHARACTERISTIC_ADVINTERVAL_UUID			{0x95,0xe2,0xed,0xeb,0x1b,0xa0,0x39,0x8a,0xdf,0x4b,0xd3,0x8e,0x03,0x75,0xc8,0xa3}
#define EDDY_CHARACTERISTIC_RADIOTXPOWER_UUID			{0x95,0xe2,0xed,0xeb,0x1b,0xa0,0x39,0x8a,0xdf,0x4b,0xd3,0x8e,0x04,0x75,0xc8,0xa3}
#define EDDY_CHARACTERISTIC_ADVTXPOWER_UUID				{0x95,0xe2,0xed,0xeb,0x1b,0xa0,0x39,0x8a,0xdf,0x4b,0xd3,0x8e,0x05,0x75,0xc8,0xa3}
#define EDDY_CHARACTERISTIC_LOCKSTATE_UUID				{0x95,0xe2,0xed,0xeb,0x1b,0xa0,0x39,0x8a,0xdf,0x4b,0xd3,0x8e,0x06,0x75,0xc8,0xa3}
#define EDDY_CHARACTERISTIC_UNLOCK_UUID					{0x95,0xe2,0xed,0xeb,0x1b,0xa0,0x39,0x8a,0xdf,0x4b,0xd3,0x8e,0x07,0x75,0xc8,0xa3}
#define EDDY_CHARACTERISTIC_ADVSLOTDATA_UUID			{0x95,0xe2,0xed,0xeb,0x1b,0xa0,0x39,0x8a,0xdf,0x4b,0xd3,0x8e,0x0a,0x75,0xc8,0xa3}
#define EDDY_CHARACTERISTIC_REMAINCONNECTABLE_UUID		{0x95,0xe2,0xed,0xeb,0x1b,0xa0,0x39,0x8a,0xdf,0x4b,0xd3,0x8e,0x0c,0x75,0xc8,0xa3}

#define ATT_DECL_CHAR() 			                	{ATT_DECL_CHARACTERISTIC_128, PERM(RD, ENABLE), 0, 0}
#define ATT_DECL_CHAR_UUID_16(uuid, perm, max_length) 	{uuid, perm, max_length, PERM(RI,ENABLE) | PERM(UUID_LEN,UUID_16)}
#define ATT_DECL_CHAR_UUID_32(uuid, perm, max_length)   {uuid, perm, max_length, PERM(RI,ENABLE) | PERM(UUID_LEN,UUID_32)}
#define ATT_DECL_CHAR_UUID_128(uuid, perm, max_length)  {uuid, perm, max_length, PERM(RI,ENABLE) | PERM(UUID_LEN,UUID_128)}
#define ATT_DECL_CHAR_CCC() 			                {ATT_DESC_CLIENT_CHAR_CFG_128, PERM(RD, ENABLE) | PERM(WRITE_REQ, ENABLE), 0, PERM(RI, ENABLE)}
#define ATT_DECL_CHAR_USER_DESC(max_length) 	        {ATT_DESC_CHAR_USER_DESC_128, PERM(RD, ENABLE), max_length, PERM(RI, ENABLE)}


enum eddy_idx_att {
	/* Capabilities Characteristic */
	EDDY_IDX_CAPABILITIES_VALUE_CHAR,
	EDDY_IDX_CAPABILITIES_VALUE_VAL,
	EDDY_IDX_CAPABILITIES_VALUE_CCC,
	EDDY_IDX_CAPABILITIES_VALUE_USR_DSCP,

	/* Active Slot Characteristic */
	EDDY_IDX_ACTIVESLOT_VALUE_CHAR,
	EDDY_IDX_ACTIVESLOT_VALUE_VAL,
	EDDY_IDX_ACTIVESLOT_VALUE_CCC,
	EDDY_IDX_ACTIVESLOT_VALUE_USR_DSCP,

	/* Advertising Interval Characteristic */
	EDDY_IDX_ADVINTERVAL_VALUE_CHAR,
	EDDY_IDX_ADVINTERVAL_VALUE_VAL,
	EDDY_IDX_ADVINTERVAL_VALUE_CCC,
	EDDY_IDX_ADVINTERVAL_VALUE_USR_DSCP,

	/* Radio Tx Power Characteristic */
	EDDY_IDX_RADIOTXPOWER_VALUE_CHAR,
	EDDY_IDX_RADIOTXPOWER_VALUE_VAL,
	EDDY_IDX_RADIOTXPOWER_VALUE_CCC,
	EDDY_IDX_RADIOTXPOWER_VALUE_USR_DSCP,

	/* Advertising Interval Characteristic */
	EDDY_IDX_ADVTXPOWER_VALUE_CHAR,
	EDDY_IDX_ADVTXPOWER_VALUE_VAL,
	EDDY_IDX_ADVTXPOWER_VALUE_CCC,
	EDDY_IDX_ADVTXPOWER_VALUE_USR_DSCP,

	/* Lock State Characteristics */
	EDDY_IDX_LOCKSTATE_VALUE_CHAR,
	EDDY_IDX_LOCKSTATE_VALUE_VAL,
	EDDY_IDX_LOCKSTATE_VALUE_CCC,
	EDDY_IDX_LOCKSTATE_VALUE_USR_DSCP,

	/* Unlock Characteristics */
	EDDY_IDX_UNLOCK_VALUE_CHAR,
	EDDY_IDX_UNLOCK_VALUE_VAL,
	EDDY_IDX_UNLOCK_VALUE_CCC,
	EDDY_IDX_UNLOCK_VALUE_USR_DSCP,


	/* Advertising Data Characteristics */
	EDDY_IDX_ADVSLOTDATA_VALUE_CHAR,
	EDDY_IDX_ADVSLOTDATA_VALUE_VAL,
	EDDY_IDX_ADVSLOTDATA_VALUE_CCC,
	EDDY_IDX_ADVSLOTDATA_VALUE_USR_DSCP,

	/* Remain Connectable Characteristics */
	EDDY_IDX_REMAINCONNECTABLE_VALUE_CHAR,
	EDDY_IDX_REMAINCONNECTABLE_VALUE_VAL,
	EDDY_IDX_REMAINCONNECTABLE_VALUE_CCC,
	EDDY_IDX_REMAINCONNECTABLE_VALUE_USR_DSCP,

	/* Max number of characteristics */
	EDDY_IDX_NB,
};

#define EDDY_USER_DESCRIPTION_MAX_LENGTH			16
#define EDDY_CAPABILITIES_MAX_LENGTH				9
#define EDDY_LOCKSTATE_MAX_LENGTH					17
#define EDDY_REMAINCONNECTABLE_LENGTH				1
#define EDDY_ADVSLOTDATA_MAX_LENGTH					17
#define EDDY_CAPABILITIES_CHARACTERISTIC_NAME       "Capabilities"
#define EDDY_ADVINTERVAL_CHARACTERISTIC_NAME        "Advertising Interval"
#define EDDY_RADIOTXPOWER_CHARACTERISTIC_NAME		"Radio Tx Power"
#define EDDY_ADVTXPOWER_CHARACTERISTIC_NAME			"Advertised Tx Power"
#define EDDY_LOCKSTATE_CHARACTERISTIC_NAME			"Lock State"
#define EDDY_ACTIVESLOT_CHARACTERISTIC_NAME			"Active Slot"
#define EDDY_UNLOCK_CHARACTERISTIC_NAME				"Unlock"
#define EDDY_ADVSLOTDATA_CHARACTERISTIC_NAME		"Advertisment Slot Data"
#define EDDY_REMAINCONNECTABLE_CHARACTERISTIC_NAME	"Remain Connectable"


/* List of message handlers that are used by the custom service application manager */
#define EDDY_MESSAGE_HANDLER_LIST \
        DEFINE_MESSAGE_HANDLER(GATTC_READ_REQ_IND, Eddy_GATTC_ReadReqInd),\
        DEFINE_MESSAGE_HANDLER(GATTM_ADD_SVC_RSP, Eddy_GATTM_AddSvcRsp),\
		DEFINE_MESSAGE_HANDLER(GATTC_WRITE_REQ_IND, Eddy_GATTC_WriteReqInd)



struct eddy_env_tag {
	/* The value of service handle in the database of attributes in the stack */
	uint16_t start_hdl;

	uint8_t capabilities_value[EDDY_CAPABILITIES_MAX_LENGTH];
	uint8_t capabilities_cccd_value;

	uint8_t activeslot_value;
	uint8_t activeslot_cccd_value;

	uint16_t advinterval_value;
	uint8_t advinterval_cccd_value;

	uint8_t radiotxpower_value;
	uint8_t radiotxpower_cccd_value;

	uint8_t advtxpower_value;
	uint8_t advtxpower_cccd_value;

	uint8_t lockstate_value[EDDY_LOCKSTATE_MAX_LENGTH];
	uint8_t lockstate_cccd_value;

	uint8_t challenge_value[16];
	uint8_t challenge_cccd_value;

	uint8_t unlocktoken_value[16];
	uint8_t unlocktoken_cccd_value;

	uint8_t advslotdata_value[EDDY_ADVSLOTDATA_MAX_LENGTH];
	uint8_t advslotdata_cccd_value;
	uint8_t advslotdata_length;

	uint8_t remainconnectable_value;
	uint8_t remainconnectable_cccd_value;



};



extern struct eddy_env_tag eddy_env;

/* ----------------------------------------------------------------------------
 * Function prototype definitions
 * --------------------------------------------------------------------------*/
extern void EddyService_Env_Initialize(void);
extern void EddyService_ServiceAdd(void);
extern int Eddy_GATTM_AddSvcRsp(ke_msg_id_t const msgid,
		struct gattm_add_svc_rsp const *param, ke_task_id_t const dest_id,
		ke_task_id_t const src_id);
extern int Eddy_GATTC_ReadReqInd(ke_msg_id_t const msg_id,
		struct gattc_read_req_ind const *param, ke_task_id_t const dest_id,
		ke_task_id_t const src_id);

extern int Eddy_GATTC_WriteReqInd(ke_msg_id_t const msg_id,
                      struct gattc_write_req_ind const *param,
                      ke_task_id_t const dest_id, ke_task_id_t const src_id);

/* ----------------------------------------------------------------------------
 * Close the 'extern "C"' block
 * ------------------------------------------------------------------------- */
#ifdef __cplusplus
}
#endif

#endif /* BLE_CUSTOM_H */
